import { Store } from 'vuex'

export default {
  registerStorePlugin: (
    logInMutation: string,
    logOutMutation: string,
    isLoggedInGetterName: string,
    memberAuthorityGetterName: string
  ): ((store: Store<S>) => {}) => {},

  set loggedInGetterName(value: string): void {},
  get loggedInGetterName(): string {
    return ''
  },

  set logInMutationType(value: string): void {},
  get logInMutationType(): string {
    return ''
  },

  set logOutMutationType(value: string): void {},
  get logOutMutationType(): string {
    return ''
  },

  set memberAuthorityGetterName(value: string): void {},
  get memberAuthorityGetterName(): string {
    return ''
  },
}
